<?php
/**
 * 2024 Addify
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    Addify
 *  @copyright 2024 Addify
 *  @license   http://opensource.org/licenses/afl-3.0.php
 */

if (!defined('_PS_VERSION_')) {
    exit;
}

if (!defined('_MYSQL_ENGINE_'))

define('_MYSQL_ENGINE_', 'MyISAM');

include_once dirname(__FILE__) . '/classes/AddifyRedirectUrl.php';
include_once dirname(__FILE__) . '/classes/AddifyAllBrokenLinks.php';

class Addifyadvanceurls extends Module

{

	public function __construct()

	{

		$this->name = 'addifyadvanceurls';

		$this->tab = 'seo';

		$this->version = '2.0.0';

		$this->author = 'Addify';
		
        $this->module_key = '68a855e73e1b3e54ddd8d12ccd72700d';

		$this->bootstrap = true;

		parent::__construct();

		$this->displayName = $this->l('Addify Advance URLs');

		$this->description = $this->l('Addify Advance URLs makes Seo friendly URLs by removing numeric IDs, 404s and Duplicate URLs.');

        $this->confirmUninstall = $this->l('Are you sure you want to uninstall the module?');

        $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);

		$this->tabClass = 'AdminAddifyadvanceurls';

	}



	public function install()

	{
		Configuration::updateValue('ADDIFY_PRETTY_ATTR_ID', true);
		Configuration::updateValue('ADDIFY_PRETTY_ID', true);
		Configuration::updateValue('ADDIFY_PRETTY_REDIRECT_PRD_URLS', true);
		Configuration::updateValue('ADDIFY_PRETTY_ID_CATEGORY', true);
		Configuration::updateValue('ADDIFY_PRETTY_REDIRECT_CATEGORY_URLS', true);
		Configuration::updateValue('ADDIFY_PRETTY_ID_CMS', true);
		Configuration::updateValue('ADDIFY_PRETTY_REDIRECT_CMS_URLS', true);
		Configuration::updateValue('ADDIFY_PRETTY_ID_CMS_CAT', true);
		Configuration::updateValue('ADDIFY_PRETTY_REDIRECT_CMS_CAT_URLS', true);
		Configuration::updateValue('ADDIFY_PRETTY_ID_BRANDS', true);
		Configuration::updateValue('ADDIFY_PRETTY_REDIRECT_BRANDS_URLS', true);
		Configuration::updateValue('ADDIFY_PRETTY_ID_SUPPLIER', true);
		Configuration::updateValue('ADDIFY_PRETTY_REDIRECT_SUPPLIER_URLS', true);
		include(dirname(__FILE__).'/sql/install.php');
		return (parent::install()
				&& $this->installTab()
				&& $this->changecontrollers()
				&& $this->flushCache()
				&& $this->initTokenSecure()
				&& $this->registerHook('displayBackOfficeHeader')
				&& $this->registerHook('displayHeader'));
	}



	public function uninstall()

	{
		return ($this->cancelchanges()

				&& parent::uninstall()

				&& $this->uninstallTab()

				&& $this->unregisterHook('displayBackOfficeHeader')
		
				&& $this->unregisterHook('displayHeader')

				&& $this->flushCache());

	}

	public function uninstallTabOldVersion()
	{
		$id_tab = (int)Tab::getIdFromClassName('AdminAddifyadvanceurls');
		if ($id_tab) {
			$tab = new Tab($id_tab);
			return $tab->delete();
		}
		return true;
	}

	public function initTokenSecure()
    {
        $now = date('Y-m-d H:i:s');
		$time_string = strtotime($now);
		$time_string = md5(_COOKIE_KEY_.$time_string);
		Configuration::updateValue('ADDIFY_PRETTY_TOKEN', $time_string);
        return true;
    }



	public function installTab()

	{

		$tab = new Tab();
		$tab->active = 1;
		$tab->class_name = $this->tabClass;
		$tab->name = array();
		foreach (Language::getLanguages(true) as $lang) {
			$tab->name[$lang['id_lang']] = 'Duplicate URLs';
		}
		$tab->id_parent = (Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=')) ? (int)Tab::getIdFromClassName('AdminParentMeta') : (int)Tab::getIdFromClassName('AdminParentPreferences');
		$tab->module = $this->name;
		$tab->add();

		$tab = new Tab();
		$tab->active = 1;
		$tab->class_name = 'AdminAddifyManageRedirect';
		$tab->name = array();
		foreach (Language::getLanguages(true) as $lang) {
			$tab->name[$lang['id_lang']] = 'URL Redirect';
		}
		$tab->id_parent = (Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=')) ? (int)Tab::getIdFromClassName('AdminParentMeta') : (int)Tab::getIdFromClassName('AdminParentPreferences');
		$tab->module = $this->name;
		$tab->add();

		return true;

	}

	

	public function uninstallTab()

	{

		$id_tab = (int)Tab::getIdFromClassName('AdminAddifyadvanceurls');

		
		if ($id_tab) {
			$tab = new Tab($id_tab);
			$tab->delete();
			$id_tab = (int)Tab::getIdFromClassName('AdminAddifyManageRedirect');
			$tab = new Tab($id_tab);
			$tab->delete();
		}
		return true;
	}

	

	public function getContent()

    {
    	$warning= '';
		$this->html_performance_link = $this->display(__FILE__, 'views/templates/admin/performance_link.tpl');
		$this->html_seo_link = $this->display(__FILE__, 'views/templates/admin/seo_link.tpl');
		$warning .= $this->displayWarning($this->l('1. Turn OFF the cache for the testing of new URLs, you can turn it ON later.'));
		$warning .= $this->displayWarning($this->l('2. Please view Settings in ').$this->html_seo_link);
		$warning .= $this->displayWarning($this->l('3. Please make sure overrides are NOT disabled in ').$this->html_performance_link);
		return $warning . $this->postProcess() . $this->_displayForm();
	}

	private function _displayForm()
    {
        $type = (Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=') == true) ? 'ps17' : 'ps16';
		$module_tab = Tools::getValue('tab_module');
        $this->context->smarty->assign([
            '_ps_ver' => $type,
            'general_form' => $this->generalForm(),
			'redirects_form' => $this->redirectsForm(),
			'module_tab' => $module_tab
        ]);

        return $this->display(__FILE__, 'views/templates/admin/configure.tpl');
    }



	public function postProcess()
    {	
    	if (Tools::isSubmit('submit' . $this->name)) {

    		$token = Configuration::get('ADDIFY_PRETTY_TOKEN');
			if (!$token || empty($token)) {
				$this->initTokenSecure();
			}
			$redirectCmsCatUrls = (int) Tools::getValue('ADDIFY_PRETTY_REDIRECT_CMS_CAT_URLS');
			//Product URLs
			$putIdAttr = (int) Tools::getValue('ADDIFY_PRETTY_ATTR_ID');
			$productRoute = $this->getRoute('product_rule');
			if ($putIdAttr > 0) {
				if (strpos($productRoute, '{id_product_attribute:-}') !== false) {
					$productRoute = str_replace('{id_product_attribute:-}', '', $productRoute);
					Configuration::updateValue('PS_ROUTE_product_rule', $productRoute);
					Cache::clean('Module::isInstalled' . $this->name);
				}
				elseif (strpos($productRoute, '{id_product_attribute}') !== false) {
					$productRoute = str_replace('{id_product_attribute}-', '', $productRoute);
					Configuration::updateValue('PS_ROUTE_product_rule', $productRoute);
					Cache::clean('Module::isInstalled' . $this->name);
				}
			}
			elseif ($putIdAttr <= 0 && (strpos($productRoute, '{id_product_attribute:-}') === false)) {
				$productRoute = str_replace('{rewrite}', '{id_product_attribute:-}{rewrite}', $productRoute);
				Configuration::updateValue('PS_ROUTE_product_rule', $productRoute);
				Cache::clean('Module::isInstalled' . $this->name);
			}
			
			$idProductUrl = (int) Tools::getValue('ADDIFY_PRETTY_ID');
			if ($idProductUrl > 0) {
				if (strpos($productRoute, '{id:-}') !== false) {
					$productRoute = str_replace('{id:-}', '', $productRoute);
					Configuration::updateValue('PS_ROUTE_product_rule', $productRoute);
					Cache::clean('Module::isInstalled' . $this->name);
				}
				elseif (strpos($productRoute, '{id}-') !== false) {
					$productRoute = str_replace('{id}-', '', $productRoute);
					Configuration::updateValue('PS_ROUTE_product_rule', $productRoute);
					Cache::clean('Module::isInstalled' . $this->name);
				}
			}
			elseif ($idProductUrl <= 0 && strpos($productRoute, '{id:-}') === false && strpos($productRoute, '{id}') === false) {
				if (strpos($productRoute, '{id_product_attribute:-}') !== false) {
					$productRoute = str_replace('{id_product_attribute:-}', '{id:-}{id_product_attribute:-}', $productRoute);
					Configuration::updateValue('PS_ROUTE_product_rule', $productRoute);
					Cache::clean('Module::isInstalled' . $this->name);
				}
				else {
					$productRoute = str_replace('{rewrite}', '{id:-}{rewrite}', $productRoute);
					Configuration::updateValue('PS_ROUTE_product_rule', $productRoute);
					Cache::clean('Module::isInstalled' . $this->name);
				}
			}
			$redirectProductUrls = (int) Tools::getValue('ADDIFY_PRETTY_REDIRECT_PRD_URLS');
			$overrideFileOfPrdCont = _PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'ProductController.php';
			if ($redirectProductUrls > 0 && (int)Tools::file_exists_no_cache($overrideFileOfPrdCont) <= 0) {
				Tools::copy(_PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'includes'.DIRECTORY_SEPARATOR.'ProductController.php', $overrideFileOfPrdCont);
				Cache::clean('Module::isInstalled' . $this->name);
				PrestashopAutoload::getInstance()->generateIndex();
			}
			elseif ($redirectProductUrls <= 0) {
				Tools::deleteFile(_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'ProductController.php');
				Cache::clean('Module::isInstalled' . $this->name);
				PrestashopAutoload::getInstance()->generateIndex();
			}
			//Category URLs
			$categoryIdResult = (int) Tools::getValue('ADDIFY_PRETTY_ID_CATEGORY');
			$categoryRoute = $this->getRoute('category_rule');
			if ($categoryIdResult > 0) {
				if (strpos($categoryRoute, '{id:-}') !== false) {
					$categoryRoute = str_replace('{id:-}', '', $categoryRoute);
					Configuration::updateValue('PS_ROUTE_category_rule', $categoryRoute);
					Cache::clean('Module::isInstalled' . $this->name);
				}
				elseif (strpos($categoryRoute, '{id}-') !== false) {
					$categoryRoute = str_replace('{id}-', '', $categoryRoute);
					Configuration::updateValue('PS_ROUTE_category_rule', $categoryRoute);
					Cache::clean('Module::isInstalled' . $this->name);
				}
			}
			elseif ($categoryIdResult <= 0 && strpos($categoryRoute, '{id:-}') === false && strpos($categoryRoute, '{id}') === false) {
				$categoryRoute = str_replace('{rewrite}', '{id:-}{rewrite}', $categoryRoute);
				Configuration::updateValue('PS_ROUTE_category_rule', $categoryRoute);
				Cache::clean('Module::isInstalled' . $this->name);
			}
			$redirectCategoryUrls = (int) Tools::getValue('ADDIFY_PRETTY_REDIRECT_CATEGORY_URLS');
			$overrideFileOfCatCont = _PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'CategoryController.php';
			if ($redirectCategoryUrls > 0 && (int)Tools::file_exists_no_cache($overrideFileOfCatCont) <= 0) {
				Tools::copy(_PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'override'.DIRECTORY_SEPARATOR.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'CategoryController.php', $overrideFileOfCatCont);
				Cache::clean('Module::isInstalled' . $this->name);
				PrestashopAutoload::getInstance()->generateIndex();
			}
			elseif ($redirectCategoryUrls <= 0) {
				Tools::deleteFile(_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'CategoryController.php');
				Cache::clean('Module::isInstalled' . $this->name);
				PrestashopAutoload::getInstance()->generateIndex();
			}
			//CMS URLs
			$cmsIdResult = (int) Tools::getValue('ADDIFY_PRETTY_ID_CMS');
			$cmsRoute = $this->getRoute('cms_rule');
			if ($cmsIdResult > 0) {
				if (strpos($cmsRoute, '{id:-}') !== false) {
					$cmsRoute = str_replace('{id:-}', '', $cmsRoute);
					Configuration::updateValue('PS_ROUTE_cms_rule', $cmsRoute);
					Cache::clean('Module::isInstalled' . $this->name);
				}
				elseif (strpos($cmsRoute, '{id}-') !== false) {
					$cmsRoute = str_replace('{id}-', '', $cmsRoute);
					Configuration::updateValue('PS_ROUTE_cms_rule', $cmsRoute);
					Cache::clean('Module::isInstalled' . $this->name);
				}
			}
			elseif ($cmsIdResult <= 0 && strpos($cmsRoute, '{id:-}') === false && strpos($cmsRoute, '{id}') === false) {
				$cmsRoute = str_replace('{rewrite}', '{id:-}{rewrite}', $cmsRoute);
				Configuration::updateValue('PS_ROUTE_cms_rule', $cmsRoute);
				Cache::clean('Module::isInstalled' . $this->name);
			}
			$redirectCmsUrls = (int) Tools::getValue('ADDIFY_PRETTY_REDIRECT_CMS_URLS');
			$overrideFileOfCMSCont = _PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'CmsController.php';
			if ($redirectCmsUrls > 0 && (int)Tools::file_exists_no_cache($overrideFileOfCMSCont) <= 0) {
				Tools::copy(_PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'override'.DIRECTORY_SEPARATOR.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'CmsController.php', $overrideFileOfCMSCont);
				Cache::clean('Module::isInstalled' . $this->name);
				PrestashopAutoload::getInstance()->generateIndex();
			}
			//CMS Category URLs
			$cmsCatIdResult = (int) Tools::getValue('ADDIFY_PRETTY_ID_CMS_CAT');
			$cmsCatRoute = $this->getRoute('cms_category_rule');
			if ($cmsCatIdResult > 0) {
				if (strpos($cmsCatRoute, '{id:-}') !== false) {
					$cmsCatRoute = str_replace('{id:-}', '', $cmsCatRoute);
					Configuration::updateValue('PS_ROUTE_cms_category_rule', $cmsCatRoute);
					Cache::clean('Module::isInstalled' . $this->name);
				}
				elseif (strpos($cmsCatRoute, '{id}-') !== false) {
					$cmsCatRoute = str_replace('{id}-', '', $cmsCatRoute);
					Configuration::updateValue('PS_ROUTE_cms_category_rule', $cmsCatRoute);
					Cache::clean('Module::isInstalled' . $this->name);
				}
			}
			elseif ($cmsCatIdResult <= 0 && strpos($cmsCatRoute, '{id:-}') === false && strpos($cmsCatRoute, '{id}') === false) {
				$cmsCatRoute = str_replace('{rewrite}', '{id:-}{rewrite}', $cmsCatRoute);
				Configuration::updateValue('PS_ROUTE_cms_category_rule', $cmsCatRoute);
				Cache::clean('Module::isInstalled' . $this->name);
			}
			//Brands URLs
			$brandsIdResult = (int) Tools::getValue('ADDIFY_PRETTY_ID_BRANDS');
			$brandsRoute = $this->getRoute('manufacturer_rule');
			if ($brandsIdResult > 0) {
				if (strpos($brandsRoute, '{id:-}') !== false) {
					$brandsRoute = str_replace('{id:-}', '', $brandsRoute);
					Configuration::updateValue('PS_ROUTE_manufacturer_rule', $brandsRoute);
					Cache::clean('Module::isInstalled' . $this->name);
				}
				elseif (strpos($brandsRoute, '{id}-') !== false) {
					$brandsRoute = str_replace('{id}-', '', $brandsRoute);
					Configuration::updateValue('PS_ROUTE_manufacturer_rule', $brandsRoute);
					Cache::clean('Module::isInstalled' . $this->name);
				}
			}
			elseif ($brandsIdResult <= 0 && strpos($brandsRoute, '{id:-}') === false && strpos($brandsRoute, '{id}') === false) {
				$brandsRoute = str_replace('{rewrite}', '{id:-}{rewrite}', $brandsRoute);
				Configuration::updateValue('PS_ROUTE_manufacturer_rule', $brandsRoute);
				Cache::clean('Module::isInstalled' . $this->name);
			}
			$redirectBrandsUrls = (int) Tools::getValue('ADDIFY_PRETTY_REDIRECT_BRANDS_URLS');
			$overrideFileOfBrandsCont = _PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'ManufacturerController.php';
			if ($redirectBrandsUrls > 0 && (int)Tools::file_exists_no_cache($overrideFileOfBrandsCont) <= 0) {
				Tools::copy(_PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'override'.DIRECTORY_SEPARATOR.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'ManufacturerController.php', $overrideFileOfBrandsCont);
				Cache::clean('Module::isInstalled' . $this->name);
				PrestashopAutoload::getInstance()->generateIndex();
			}
			elseif ($redirectBrandsUrls <= 0) {
				Tools::deleteFile(_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'ManufacturerController.php');
				Cache::clean('Module::isInstalled' . $this->name);
				PrestashopAutoload::getInstance()->generateIndex();
			}
			//SUPPLIER URLs
			$supplierIdResult = (int) Tools::getValue('ADDIFY_PRETTY_ID_SUPPLIER');
			$supplierRoute = $this->getRoute('supplier_rule');
			if ($supplierIdResult > 0) {
				if (strpos($supplierRoute, '{id:-}') !== false) {
					$supplierRoute = str_replace('{id:-}', '', $supplierRoute);
					Configuration::updateValue('PS_ROUTE_supplier_rule', $supplierRoute);
					Cache::clean('Module::isInstalled' . $this->name);
				}
				elseif (strpos($supplierRoute, '{id}-') !== false) {
					$supplierRoute = str_replace('{id}-', '', $supplierRoute);
					Configuration::updateValue('PS_ROUTE_supplier_rule', $supplierRoute);
					Cache::clean('Module::isInstalled' . $this->name);
				}
			}
			elseif ($supplierIdResult <= 0 && strpos($supplierRoute, '{id:-}') === false && strpos($supplierRoute, '{id}') === false) {
				$supplierRoute = str_replace('{rewrite}', '{id:-}{rewrite}', $supplierRoute);
				Configuration::updateValue('PS_ROUTE_supplier_rule', $supplierRoute);
				Cache::clean('Module::isInstalled' . $this->name);
			}
			$redirectSupplierUrls = (int) Tools::getValue('ADDIFY_PRETTY_REDIRECT_SUPPLIER_URLS');
			$overrideFileOfSupplierCont = _PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'SupplierController.php';
			if ($redirectSupplierUrls > 0 && (int)Tools::file_exists_no_cache($overrideFileOfSupplierCont) <= 0) {
				Tools::copy(_PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'override'.DIRECTORY_SEPARATOR.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'SupplierController.php', $overrideFileOfSupplierCont);
				Cache::clean('Module::isInstalled' . $this->name);
				PrestashopAutoload::getInstance()->generateIndex();
			}
			elseif ($redirectSupplierUrls <= 0) {
				Tools::deleteFile(_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'SupplierController.php');
				Cache::clean('Module::isInstalled' . $this->name);
				PrestashopAutoload::getInstance()->generateIndex();
			}
		
			Configuration::updateValue('ADDIFY_PRETTY_ISO', (int) Tools::getValue('ADDIFY_PRETTY_ISO'));
			Configuration::updateValue('ADDIFY_PRETTY_ATTR_ALIAS', (int) Tools::getValue('ADDIFY_PRETTY_ATTR_ALIAS'));
			Configuration::updateValue('ADDIFY_PRETTY_ATTR_ID', (int) $putIdAttr);
			Configuration::updateValue('ADDIFY_PRETTY_ID', (int) $idProductUrl);
			Configuration::updateValue('ADDIFY_PRETTY_REDIRECT_PRD_URLS', (int) $redirectProductUrls);
			Configuration::updateValue('ADDIFY_PRETTY_REDIRECT_CATEGORY_URLS', (int) $redirectCategoryUrls);
			Configuration::updateValue('ADDIFY_PRETTY_ID_CATEGORY', (int) $categoryIdResult);
			Configuration::updateValue('ADDIFY_PRETTY_REDIRECT_CMS_URLS', (int) $redirectCmsUrls);
			Configuration::updateValue('ADDIFY_PRETTY_ID_CMS', (int) $cmsIdResult);
			Configuration::updateValue('ADDIFY_PRETTY_REDIRECT_CMS_CAT_URLS', (int) $redirectCmsCatUrls);
			Configuration::updateValue('ADDIFY_PRETTY_ID_CMS_CAT', (int) $cmsCatIdResult);
			Configuration::updateValue('ADDIFY_PRETTY_REDIRECT_BRANDS_URLS', (int) $redirectBrandsUrls);
			Configuration::updateValue('ADDIFY_PRETTY_ID_BRANDS', (int) $brandsIdResult);
			Configuration::updateValue('ADDIFY_PRETTY_REDIRECT_SUPPLIER_URLS', (int) $redirectSupplierUrls);
			Configuration::updateValue('ADDIFY_PRETTY_ID_SUPPLIER', (int) $supplierIdResult);

			$this->flushCache();

			Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
    	}
    	elseif (Tools::isSubmit('submitRedirects'. $this->name)) {
    		$redirect_404 = Tools::getValue('ADDIFY_REDIRECT_404');
            if ($redirect_404 == 2) {
                $cms_pages = Tools::getValue('cms_pages');
                Configuration::updateValue('ADDIFY_REDIRECT_404_CMS', $cms_pages);
            }
            Configuration::updateValue('ADDIFY_REDIRECT_404', $redirect_404);
            Configuration::updateValue('ADDIFY_404_URL', Tools::getValue('ADDIFY_404_URL'));
            Configuration::updateValue('ADDIFY_URL_REDIRECTION', Tools::getValue('ADDIFY_URL_REDIRECTION'));


            $this->clear404HtaccessFile();

            $this->add404UrlsToHtaccessFile();

            $this->flushCache();
           
            Tools::redirectAdmin($this->context->link->getAdminLink('AdminModules', true) . '&conf=4&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name);
    	}
		
	}


	public function generalForm()
    {
		$fields_form = [
            'form' => [
                'legend' => [
                    'title' => $this->l('URL Cleaner Settings'),
                    'icon' => 'icon-cogs',
                ],
                'tabs' => [
                    'general' => $this->l('General Settings'),
                    'product_urls' => $this->l('Product URLs'),
					'category_urls' => $this->l('Category URLs'),
					'supplier_urls' => $this->l('Supplier URLs'),
					'brand_urls' => $this->l('Brands URLs'),
					'cms_urls' => $this->l('CMS URLs'),
					'cms_cat_urls' => $this->l('CMS Category URLs'),
					
					
                ],
                'input' => [
                    [
                        'type' => 'switch',
                        'label' => $this->l('Remove Language ISO from Default Language?'),
                        'name' => 'ADDIFY_PRETTY_ISO',
                        'required' => false,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'pretty_iso_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'pretty_iso_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                        'tab' => 'general',
                    ],
					[
                        'type' => 'switch',
                        'label' => $this->l('Remove ID From URLs?'),
                        'name' => 'ADDIFY_PRETTY_ID',
                        'required' => false,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'pretty_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'pretty_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                        'tab' => 'product_urls',
                    ],
					[
                        'type' => 'switch',
                        'label' => $this->l('Remove ID Attribute?'),
                        'name' => 'ADDIFY_PRETTY_ATTR_ID',
                        'required' => false,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'pretty_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'pretty_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                        'tab' => 'product_urls',
                    ],
					[
                        'type' => 'switch',
                        'label' => $this->l('Remove Attribute Alias?'),
                        'name' => 'ADDIFY_PRETTY_ATTR_ALIAS',
                        'required' => false,
						'desc' => $this->l('For example the hash content like #/1-size-s/8-color-white'),
                        'class' => 't',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'pretty_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'pretty_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                        'tab' => 'product_urls',
                    ],
					[
                        'type' => 'switch',
                        'label' => $this->l('Redirect OLD URLs to NEW?'),
                        'name' => 'ADDIFY_PRETTY_REDIRECT_PRD_URLS',
                        'required' => false,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'pretty_redi_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'pretty_redi_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                        'tab' => 'product_urls',
                    ],
					[
						'type' => 'text',
                        'label' => $this->l('Product URL Structure'),
                        'name' => 'ADDIFY_PRETTY_PRODUCT_URL_SCHEME',
						'disabled' => true,
						'desc' => $this->l('Can be modified in "Shop Parameters - Traffic & SEO" settings'),
						'tab' => 'product_urls',
					],
					[
                        'type' => 'switch',
                        'label' => $this->l('Remove ID From URLs?'),
                        'name' => 'ADDIFY_PRETTY_ID_CATEGORY',
                        'required' => false,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'pretty_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'pretty_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                        'tab' => 'category_urls',
                    ],
					[
                        'type' => 'switch',
                        'label' => $this->l('Redirect OLD URLs to NEW?'),
                        'name' => 'ADDIFY_PRETTY_REDIRECT_CATEGORY_URLS',
                        'required' => false,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'pretty_redi_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'pretty_redi_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                        'tab' => 'category_urls',
                    ],
					[
						'type' => 'text',
                        'label' => $this->l('Category URL Structure'),
                        'name' => 'ADDIFY_PRETTY_CATEGORY_URL_SCHEME',
						'disabled' => true,
						'desc' => $this->l('Can be modified in "Shop Parameters - Traffic & SEO" settings'),
						'tab' => 'category_urls',
					],
					[
                        'type' => 'switch',
                        'label' => $this->l('Remove ID From URLs?'),
                        'name' => 'ADDIFY_PRETTY_ID_CMS',
                        'required' => false,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'pretty_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'pretty_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                        'tab' => 'cms_urls',
                    ],
					[
                        'type' => 'switch',
                        'label' => $this->l('Redirect OLD URLs to NEW?'),
                        'name' => 'ADDIFY_PRETTY_REDIRECT_CMS_URLS',
                        'required' => false,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'pretty_redi_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'pretty_redi_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                        'tab' => 'cms_urls',
                    ],
					[
						'type' => 'text',
                        'label' => $this->l('CMS URL Structure'),
                        'name' => 'ADDIFY_PRETTY_CMS_URL_SCHEME',
						'disabled' => true,
						'desc' => $this->l('Can be modified in "Shop Parameters - Traffic & SEO" settings'),
						'tab' => 'cms_urls',
					],
					[
                        'type' => 'switch',
                        'label' => $this->l('Remove ID From URLs?'),
                        'name' => 'ADDIFY_PRETTY_ID_CMS_CAT',
                        'required' => false,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'pretty_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'pretty_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                        'tab' => 'cms_cat_urls',
                    ],
					[
                        'type' => 'switch',
                        'label' => $this->l('Redirect OLD URLs to NEW?'),
                        'name' => 'ADDIFY_PRETTY_REDIRECT_CMS_CAT_URLS',
                        'required' => false,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'pretty_redi_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'pretty_redi_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                        'tab' => 'cms_cat_urls',
                    ],
					[
						'type' => 'text',
                        'label' => $this->l('CMS Category URL Structure'),
                        'name' => 'ADDIFY_PRETTY_CMS_CAT_URL_SCHEME',
						'disabled' => true,
						'desc' => $this->l('Can be modified in "Shop Parameters - Traffic & SEO" settings'),
						'tab' => 'cms_cat_urls',
					],
					[
                        'type' => 'switch',
                        'label' => $this->l('Remove ID From URLs?'),
                        'name' => 'ADDIFY_PRETTY_ID_BRANDS',
                        'required' => false,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'pretty_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'pretty_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                        'tab' => 'brand_urls',
                    ],
					[
                        'type' => 'switch',
                        'label' => $this->l('Redirect OLD URLs to NEW?'),
                        'name' => 'ADDIFY_PRETTY_REDIRECT_BRANDS_URLS',
                        'required' => false,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'pretty_redi_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'pretty_redi_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                        'tab' => 'brand_urls',
                    ],
					[
						'type' => 'text',
                        'label' => $this->l('Manufacturer URL Structure'),
                        'name' => 'ADDIFY_PRETTY_BRANDS_URL_SCHEME',
						'disabled' => true,
						'desc' => $this->l('Can be modified in "Shop Parameters - Traffic & SEO" settings'),
						'tab' => 'brand_urls',
					],
					[
                        'type' => 'switch',
                        'label' => $this->l('Remove ID From URLs?'),
                        'name' => 'ADDIFY_PRETTY_ID_SUPPLIER',
                        'required' => false,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'pretty_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'pretty_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                        'tab' => 'supplier_urls',
                    ],
					[
                        'type' => 'switch',
                        'label' => $this->l('Redirect OLD URLs to NEW?'),
                        'name' => 'ADDIFY_PRETTY_REDIRECT_SUPPLIER_URLS',
                        'required' => false,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'pretty_redi_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'pretty_redi_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                        'tab' => 'supplier_urls',
                    ],
					[
						'type' => 'text',
                        'label' => $this->l('SUPPLIER URL Structure'),
                        'name' => 'ADDIFY_PRETTY_SUPPLIER_URL_SCHEME',
						'disabled' => true,
						'desc' => $this->l('Can be modified in "Shop Parameters - Traffic & SEO" settings'),
						'tab' => 'supplier_urls',
					],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];
		$helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->module = $this;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submit' . $this->name;
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false) . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
        $helper->tpl_vars = [
            'uri' => $this->getPathUri(),
            'fields_value' => $this->getConfigFieldsValues(),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
        ];

        return $helper->generateForm([$fields_form]);
    }
	
	public function getConfigFieldsValues()
    {
        $PRD_URL_RULE = $this->getRoute('product_rule');
        $fields = [];
        
		$fields['ADDIFY_PRETTY_ISO'] = (int) Configuration::get('ADDIFY_PRETTY_ISO');
		$fields['ADDIFY_PRETTY_ATTR_ALIAS'] = (int) Configuration::get('ADDIFY_PRETTY_ATTR_ALIAS');
		$fields['ADDIFY_PRETTY_PRODUCT_URL_SCHEME'] = $PRD_URL_RULE;
		$fields['ADDIFY_PRETTY_ATTR_ID'] = (int) Configuration::get('ADDIFY_PRETTY_ATTR_ID');
		$fields['ADDIFY_PRETTY_ID'] = (int) Configuration::get('ADDIFY_PRETTY_ID');
		$fields['ADDIFY_PRETTY_REDIRECT_PRD_URLS'] = (int) Configuration::get('ADDIFY_PRETTY_REDIRECT_PRD_URLS');
		$fields['ADDIFY_PRETTY_CATEGORY_URL_SCHEME'] = $this->getRoute('category_rule');
		$fields['ADDIFY_PRETTY_ID_CATEGORY'] = (int) Configuration::get('ADDIFY_PRETTY_ID_CATEGORY');
		$fields['ADDIFY_PRETTY_REDIRECT_CATEGORY_URLS'] = (int) Configuration::get('ADDIFY_PRETTY_REDIRECT_CATEGORY_URLS');
		$fields['ADDIFY_PRETTY_CMS_URL_SCHEME'] = $this->getRoute('cms_rule');
		$fields['ADDIFY_PRETTY_ID_CMS'] = (int) Configuration::get('ADDIFY_PRETTY_ID_CMS');
		$fields['ADDIFY_PRETTY_REDIRECT_CMS_URLS'] = (int) Configuration::get('ADDIFY_PRETTY_REDIRECT_CMS_URLS');
		$fields['ADDIFY_PRETTY_CMS_CAT_URL_SCHEME'] = $this->getRoute('cms_category_rule');
		$fields['ADDIFY_PRETTY_ID_CMS_CAT'] = (int) Configuration::get('ADDIFY_PRETTY_ID_CMS_CAT');
		$fields['ADDIFY_PRETTY_REDIRECT_CMS_CAT_URLS'] = (int) Configuration::get('ADDIFY_PRETTY_REDIRECT_CMS_CAT_URLS');
		$fields['ADDIFY_PRETTY_BRANDS_URL_SCHEME'] = $this->getRoute('manufacturer_rule');
		$fields['ADDIFY_PRETTY_ID_BRANDS'] = (int) Configuration::get('ADDIFY_PRETTY_ID_BRANDS');
		$fields['ADDIFY_PRETTY_REDIRECT_BRANDS_URLS'] = (int) Configuration::get('ADDIFY_PRETTY_REDIRECT_BRANDS_URLS');
		$fields['ADDIFY_PRETTY_SUPPLIER_URL_SCHEME'] = $this->getRoute('supplier_rule');
		$fields['ADDIFY_PRETTY_ID_SUPPLIER'] = (int) Configuration::get('ADDIFY_PRETTY_ID_SUPPLIER');
		$fields['ADDIFY_PRETTY_REDIRECT_SUPPLIER_URLS'] = (int) Configuration::get('ADDIFY_PRETTY_REDIRECT_SUPPLIER_URLS');

        return $fields;
    }

    public function getRoute($id_route)
	{
		$default_routes = Dispatcher::getInstance()->default_routes;
		$route = Configuration::get('PS_ROUTE_'.$id_route);
        $route = (empty($route)) ? $default_routes[$id_route]['rule'] : $route;
		return $route;
	}

	public function redirectsForm()
    {
        $redirect_types = [
            ['value' => 0, 'name' => $this->l('Default')],
            ['value' => 1, 'name' => $this->l('Home Page')],
            ['value' => 2, 'name' => $this->l('CMS Page')],
            ['value' => 3, 'name' => $this->l('URL')],
            

        ];
        $redirect_trigger = [
            ['value' => 0, 'name' => $this->l('.htacces File')],
            ['value' => 1, 'name' => $this->l('Controller')],
        ];

        $fields_form = [
            'form' => [
                'tinymce' => false,
                'legend' => [
                    'title' => $this->l('URL Redirection Settings'),
                    'icon' => 'icon-cog',
                ],
                'input' => [
                	[
                        'type' => 'switch',
                        'label' => $this->l('URL Redirection'),
                        'name' => 'ADDIFY_URL_REDIRECTION',
                        'required' => false,
                        'class' => 't',
                        'is_bool' => true,
                        'values' => [
                            [
                                'id' => 'pretty_redirection_on',
                                'value' => 1,
                                'label' => $this->l('Yes'),
                            ],
                            [
                                'id' => 'pretty_redirection_off',
                                'value' => 0,
                                'label' => $this->l('No'),
                            ],
                        ],
                    ],
                    [
                        'type' => 'select',
                        'label' => $this->l('Redirect 404 Pages'),
                        'name' => 'ADDIFY_REDIRECT_404',
                        'desc' => $this->l('Redirect 404 (Page Not Found) to selected page.'),
                        'options' => [
                            'query' => $redirect_types,
                            'id' => 'value',
                            'name' => 'name',
                        ],
                    ],
                    [
						'type' => 'text',
                        'label' => $this->l('URL'),
                        'name' => 'ADDIFY_404_URL',
						'desc' => $this->l('https://www.google.com/'),
					],
                    [
                        'type' => 'aau_cms',
                        'name' => 'cms',
                        'label' => $this->l('CMS Pages'),
                    ],
                    [
                        'type' => 'aau_button',
                        'name' => 'manage_redirect_button',
                        'label' => $this->l('URL Redirect'),
                    ],
                ],
                'submit' => [
                    'title' => $this->l('Save'),
                ],
            ],
        ];

        $helper = new HelperForm();
        $helper->show_toolbar = false;
        $helper->table = $this->table;
        $helper->module = $this;
        $lang = new Language((int) Configuration::get('PS_LANG_DEFAULT'));
        $helper->default_form_language = $lang->id;
        $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') ? Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG') : 0;
        $helper->identifier = $this->identifier;
        $helper->submit_action = 'submitRedirects'.$this->name;
        $helper->currentIndex = $this->context->link->getAdminLink('AdminModules',false) . '&configure=' . $this->name .'&tab_module=redirects&module_name=' . $this->name;
        $helper->token = Tools::getAdminTokenLite('AdminModules');
		$url_redirects = $this->context->link->getAdminLink('AdminAddifyManageRedirect', true);
		
        $helper->tpl_vars = [
            'fields_value' => $this->getConfigFieldsValuesRedirects(),
            'cms_pages' => CMS::getLinks((int) $this->context->language->id),
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
            'version' => _PS_VERSION_,
            'token' => Tools::getAdminTokenLite('AdminModules'),
			'manage_link_urlredirects' => $url_redirects,
			
        ];

        return $helper->generateForm([$fields_form]);
    }
	
	public function getConfigFieldsValuesRedirects()
    {
        $return = [
            'ADDIFY_REDIRECT_404' => (int) Tools::getValue('ADDIFY_REDIRECT_404',(int) Configuration::get('ADDIFY_REDIRECT_404')),
            'ADDIFY_REDIRECT_404_CMS' => (int) Configuration::get('ADDIFY_REDIRECT_404_CMS'),
            'ADDIFY_404_URL' => Configuration::get('ADDIFY_404_URL'),
            'ADDIFY_URL_REDIRECTION' => (int) Configuration::get('ADDIFY_URL_REDIRECTION'),
        ];

        return $return;
    }


	public function changecontrollers()

	{

		if (Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=')) {

			Tools::deleteFile(_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'ProductController.php');

			Tools::deleteFile(_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'CategoryController.php');

			Tools::copy(_PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'override'.DIRECTORY_SEPARATOR.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'CategoryController.php',

						_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'CategoryController.php');

			Tools::copy(_PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'override'.DIRECTORY_SEPARATOR.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'ManufacturerController.php',

						_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'ManufacturerController.php');

			Tools::copy(_PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'override'.DIRECTORY_SEPARATOR.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'SupplierController.php',

						_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'SupplierController.php');

			Tools::copy(_PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'includes'.DIRECTORY_SEPARATOR.'ProductController.php',

						_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'ProductController.php');

			if (Tools::version_compare(_PS_VERSION_, '1.7.4.0', '<')) {

				Tools::copy(_PS_MODULE_DIR_.$this->name.DIRECTORY_SEPARATOR.'includes'.DIRECTORY_SEPARATOR.'CartController.php',

						_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'CartController.php');

			}

		}

		return true;

	}



	private function flushCache()

	{

        Tools::clearXMLCache();
        Media::clearCache();
        Tools::generateIndex();
		Tools::clearSmartyCache();

		if (true === Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=') &&
            is_callable(array('Tools', 'clearAllCache'))) {
            Tools::clearAllCache();
        }
        $timestamp = time();
		if (file_exists(_PS_CACHE_DIR_.'class_index.php') && Tools::version_compare(_PS_VERSION_, '1.7.0.0', '<')) {
			(rename(_PS_CACHE_DIR_.'class_index.php', _PS_CACHE_DIR_.$timestamp.'_class_index.php'));
		}
		if (Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=')) {
			Tools::clearSf2Cache();
		}
		if (Tools::version_compare(_PS_VERSION_, '8.0.0', '>=')) {
			Cache::clean('Module::isEnabled*');
		}
		return true;
	}



	public function cancelchanges()

	{

		Tools::deleteFile(_PS_OVERRIDE_DIR_.'classes'.DIRECTORY_SEPARATOR.'Dispatcher.php');

		Tools::deleteFile(_PS_OVERRIDE_DIR_.'classes'.DIRECTORY_SEPARATOR.'Link.php');

		if (Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=')) {

			Tools::deleteFile(_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'CategoryController.php');

			Tools::deleteFile(_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'ProductController.php');

			Tools::deleteFile(_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'ManufacturerController.php');

			Tools::deleteFile(_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'SupplierController.php');

			Tools::deleteFile(_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'CartController.php');

			Tools::deleteFile(_PS_OVERRIDE_DIR_.'controllers'.DIRECTORY_SEPARATOR.'front'.DIRECTORY_SEPARATOR.'CmsController.php');

			Tools::generateIndex();

		}

		return true;

	}

	public function add404UrlsToHtaccessFile()
    {
        $obj = new AddifyAllBrokenLinks();
        $path = _PS_ROOT_DIR_ . '/.htaccess';
        if (file_exists($path)) {
            $old_htaccess_content = Tools::file_get_contents($path);
            $redirects_collection = $obj->getAllPnfRedirects();
            $content = Tools::file_get_contents($path);
            if (preg_match('#^(.*)\#begin_addify404url.*\#end_addify404url[^\n]*(.*)$#s', $content, $m)) {
                if (!$write_fd = @fopen($path, 'w')) {
                    $this->errors[] = Tools::displayError('.htaccess not found OR not editable.');

                    return false;
                } elseif ($write_fd = @fopen($path, 'w')) {
                    fwrite($write_fd, $m[1]);
                    fwrite($write_fd, "\n#begin_addify404url\n");

                    // RedirectMatch or Redirect
                    foreach ($redirects_collection as $redirect) {
                        if (!empty($redirect['redirection_url'])) {
                            fwrite(
                                $write_fd,
                                'RedirectMatch ' . $redirect['redirect_type'] .
                                ' ' . $redirect['broken_url'] . ' ' . $redirect['redirection_url'] . "\n"
                            );
                        }
                    }

                    fwrite($write_fd, "#end_addify404url\n");
                    fclose($write_fd);
                }
            } else {
                if (!$write_fd = @fopen($path, 'w')) {
                    $this->errors[] = Tools::displayError('.htaccess not found OR not editable.');

                    return false;
                } elseif ($write_fd = @fopen($path, 'w')) {
                    fwrite($write_fd, $old_htaccess_content);
                    fwrite($write_fd, "\n#begin_addify404url\n");
                    foreach ($redirects_collection as $redirect) {
                        if (!empty($redirect['redirection_url'])) {
                            fwrite(
                                $write_fd,
                                'RedirectMatch ' . $redirect['redirect_type'] .
                                ' ' . $redirect['broken_url'] . ' ' . $redirect['redirection_url'] . "\n"
                            );
                        }
                    }
                    fwrite($write_fd, "#end_addify404url\n");
                    fclose($write_fd);
                }
            }
        } 
    }

    public function clearHtaccessFile()
    {
        $path = _PS_ROOT_DIR_ . '/.htaccess';
        $content = Tools::file_get_contents($path);
        if (preg_match('#^(.*)\#begin_addifyredirecturl.*\#end_addifyredirecturl[^\n]*(.*)$#s', $content, $m)) {
            if ($write_fd = @fopen($path, 'w')) {
                fwrite($write_fd, $m[1]);
                fclose($write_fd);
            }
        }

        return true;
    }

    public function clear404HtaccessFile()
    {
        $path = _PS_ROOT_DIR_ . '/.htaccess';
        $content = Tools::file_get_contents($path);
        if (preg_match('#^(.*)\#begin_addify404url.*\#end_addify404url[^\n]*(.*)$#s', $content, $m)) {
            if ($write_fd = @fopen($path, 'w')) {
                fwrite($write_fd, $m[1]);
                fclose($write_fd);
            }
        }

        return true;
    }


	public function addUrlsToHtaccessFile()
    {
        $obj = new AddifyRedirectUrl();
        $path = _PS_ROOT_DIR_ . '/.htaccess';
        $redirects_collection = $obj->getAllRedirects();
        $old_htaccess_content = Tools::file_get_contents($path);
        if (file_exists($path)) {
            $content = Tools::file_get_contents($path);
            if (preg_match('#^(.*)\#begin_addifyredirecturl.*\#end_addifyredirecturl[^\n]*(.*)$#s', $content, $m)) {
                if (!$write_fd = @fopen($path, 'w')) {
                    $this->errors[] = Tools::displayError('.htaccess not found OR not editable.');

                    return false;
                } elseif ($write_fd = @fopen($path, 'w')) {
                    fwrite($write_fd, $m[1]);
                    fwrite($write_fd, "\n#begin_addifyredirecturl\n");

                    // RedirectMatch or Redirect
                    foreach ($redirects_collection as $redirect) {
                        if (!empty($redirect['new_url'])) {
                            fwrite(
                                $write_fd,
                                'RedirectMatch ' . $redirect['redirect_type'] . ' ' .
                                $redirect['old_uri'] . ' ' . $redirect['new_url'] . "\n"
                            );
                        }
                    }

                    fwrite($write_fd, "#end_addifyredirecturl\n");
                    fclose($write_fd);
                }
            } else {
                if (!$write_fd = @fopen($path, 'w')) {
                    $this->errors[] = Tools::displayError('.htaccess not found OR not editable.');

                    return false;
                } elseif ($write_fd = @fopen($path, 'w')) {
                    fwrite($write_fd, $old_htaccess_content);
                    fwrite($write_fd, "\n#begin_addifyredirecturl\n");
                    foreach ($redirects_collection as $redirect) {
                        if (!empty($redirect['new_url'])) {
                            fwrite(
                                $write_fd,
                                'RedirectMatch ' . $redirect['redirect_type'] . ' ' .
                                $redirect['old_uri'] . ' ' . $redirect['new_url'] . "\n"
                            );
                        }
                    }

                    fwrite($write_fd, "#end_addifyredirecturl\n");
                    fclose($write_fd);
                }
            }
        } 
    }



	public function disable($force_all = true)

    {

		if (Tools::version_compare(_PS_VERSION_, '1.7.0.0', '>=')) {

			Tools::deleteFile(_PS_OVERRIDE_DIR_.'classes'.DIRECTORY_SEPARATOR.'Dispatcher.php');

			Tools::deleteFile(_PS_OVERRIDE_DIR_.'classes'.DIRECTORY_SEPARATOR.'Link.php');

		}

		return true;

	}

	public function notFound()
    {

        $id_lang = (int) $this->context->language->id;
        $id_shop = (int) $this->context->shop->id;
        $ssl = Tools::usingSecureMode();
        $redirect_404 = (int) Configuration::get('ADDIFY_REDIRECT_404');
        $ADDIFY_404_URL = Configuration::get('ADDIFY_404_URL');

        // 404 redirection
        if ($redirect_404 != 0 && $this->context->controller->php_self == 'pagenotfound') {

            switch ($redirect_404) {
                case 1:
                    // redirect to home page
                    Tools::redirect($this->getHomeLink($id_shop, $ssl, false));
                    break;
                case 2:
                    // redirect to cms page
                    $redirect_404_CMS = (int) Configuration::get('ADDIFY_REDIRECT_404_CMS');
                    $cms = new CMS((int) $redirect_404_CMS);
                    Tools::redirect(
                        $this->context->link->getCMSLink($cms, null, (bool) $ssl, $id_lang, $id_shop, false)
                    );
                    break;
                case 3:
                    // redirect to home page
                	if (!empty($ADDIFY_404_URL)) {
                    	Tools::redirect($ADDIFY_404_URL);
                    }
                    break;
            }
        }
    }

    protected function getHomeLink($id_shop = null, $ssl = null, $relative_protocol = false)
    {
        static $force_ssl = null;

        if ($ssl === null) {
            if ($force_ssl === null) {
                $force_ssl = (Configuration::get('PS_SSL_ENABLED') && Configuration::get('PS_SSL_ENABLED_EVERYWHERE'));
            }

            $ssl = $force_ssl;
        }

        if (Configuration::get('PS_MULTISHOP_FEATURE_ACTIVE') && $id_shop !== null) {
            $shop = new Shop($id_shop);
        } else {
            $shop = Context::getContext()->shop;
        }

        if ($relative_protocol) {
            $base = '//' . ($ssl ? $shop->domain_ssl : $shop->domain);
        } else {
            $base = (($ssl) ? 'https://' . $shop->domain_ssl : 'http://' . $shop->domain);
        }

        return $base . $shop->getBaseURI();
    }

    public function saveUrls404()
    {
        $current_url_not_found = (isset($_SERVER['HTTPS']) && $_SERVER['HTTPS'] === 'on' ? 'https' : 'http') .
            "://{$_SERVER['HTTP_HOST']}{$_SERVER['REQUEST_URI']}";

        $id_lang = (int) $this->context->language->id;
        $id_shop = (int) $this->context->shop->id;
        $ssl = Tools::usingSecureMode();
        $redirect_404 = (int) Configuration::get('ADDIFY_REDIRECT_404');
        $redirect_404_CMS = (int) Configuration::get('ADDIFY_REDIRECT_404_CMS');
        $redirect_type = 301;

        if ($redirect_404 == 1) {
        	$redirection_url = $this->getHomeLink($id_shop, $ssl, false);
        }elseif ($redirect_404 == 2) {
        	$cms = new CMS((int) $redirect_404_CMS);
        	$redirection_url =  $this->context->link->getCMSLink($cms, null, (bool) $ssl, $id_lang, $id_shop, false);
        }elseif ($redirect_404 == 3) {
        	$redirection_url = Configuration::get('ADDIFY_404_URL');
        }

        $obj = new AddifyAllBrokenLinks();
        $pnf = $obj->getPagesNotFound();

        $result = [];
        if (!empty($pnf)) {
            foreach ($pnf as $pnfs) {
                array_push($result, $pnfs['request_uri']);
            }
            $broken_url = array_map('unserialize', array_unique(array_map('serialize', $result)));
            $obj->isExists($broken_url, $redirection_url, $redirect_type);
        }
    }

    public function hookDisplayHeader()
    {

    	if (!$this->active) {
            return;
        }

        if (!Configuration::get('ADDIFY_URL_REDIRECTION')) {
            return;
        }
        
        // old url redirection
        $red_request_uri = $_SERVER['REQUEST_URI'];
      
        $request_uri_eplode = explode('/', $red_request_uri);
        $request_uri_eplode = array_filter($request_uri_eplode);
        $request_uri = end($request_uri_eplode);
        $redirect_404 = (int) Configuration::get('ADDIFY_REDIRECT_404');

        if ($redirect_404 != 0 && $this->context->controller->php_self == 'pagenotfound') {
        	
            $this->saveUrls404();
        }

        $this->addUrlsToHtaccessFile();

        $this->add404UrlsToHtaccessFile();

        $getAllRedirects = AddifyRedirectUrl::getNewRedirectUrl($request_uri);

        if (!empty($getAllRedirects)) {

            Tools::redirect($getAllRedirects);
        } else {

            $this->notFound();
        }
       
    }

	
    public function hookDisplayBackOfficeHeader()
	{
		$this->context->controller->addCSS($this->_path.'views/css/admin.css');
		$this->context->controller->addJS($this->_path.'views/js/admin.js');
	}

}