<?php
/**
 * 2024 Addify
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    Addify
 *  @copyright 2024 Addify
 *  @license   http://opensource.org/licenses/afl-3.0.php
 */

if (!defined('_PS_VERSION_')) {
    exit;
}
class AddifyAllBrokenLinks extends ObjectModel
{
    public $id_404_pages;
    public $redirect_type;
    public $broken_url;
    public $redirection_url;
    public $id_shop;
    public $id_shop_group;

    public static $definition = [
        'table' => 'addify_404_pages',
        'primary' => 'id_404_pages',
        'fields' => [
            'redirect_type' => ['type' => self::TYPE_STRING, 'required' => true],
            'id_shop' => ['type' => self::TYPE_INT],
            'id_shop_group' => ['type' => self::TYPE_INT],
            'broken_url' => ['type' => self::TYPE_STRING, 'required' => true],
            'redirection_url' => ['type' => self::TYPE_STRING, 'required' => true],
        ],
    ];

    public static function savePageNotFound($broken_url, $redirection_url, $redirect_type)
    {
        $id_shop = Context::getContext()->shop->id;
        $id_shop_group = Context::getContext()->shop->id_shop_group;
        if (!empty($broken_url)) {
            foreach ($broken_url as $broken_uri) {
                Db::getInstance()->insert('addify_404_pages', [
                    'broken_url' => pSQL($broken_uri),
                    'id_shop' => (int) $id_shop,
                    'id_shop_group' => (int) $id_shop_group,
                    'redirection_url' => pSQL($redirection_url),
                    'redirect_type' => pSQL($redirect_type),
                ]);
            }
        }

        return false;
    }

    public static function savePnf($uri, $id_shop, $id_shop_group)
    {
        Db::getInstance()->insert('addify_404_pages', [
            'broken_url' => pSQL($uri),
            'id_shop' => (int) $id_shop,
            'id_shop_group' => (int) $id_shop_group,
        ]);
    }

    public function getPagesNotFound()
    {
        $id_shop = Context::getContext()->shop->id;
        $id_shop_group = Context::getContext()->shop->id_shop_group;
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('pagenotfound');
        $sql->where('id_shop=' . (int) $id_shop);
        $sql->where('id_shop_group=' . (int) $id_shop_group);

        return Db::getInstance()->executeS($sql);
    }

    public function isExists($broken_url, $redirection_url, $redirect_type)
    {
        $id_shop = Context::getContext()->shop->id;
        $id_shop_group = Context::getContext()->shop->id_shop_group;
        $sql = new DbQuery();
        $sql->select('broken_url');
        $sql->from('addify_404_pages');
        $sql->where('id_shop=' . (int) $id_shop);
        $sql->where('id_shop_group=' . (int) $id_shop_group);
        $result = Db::getInstance()->executeS($sql);
        if (!empty($result)) {
            $unique_element = [];
            foreach ($result as $res) {
                array_push($unique_element, $res['broken_url']);
            }
            $all = array_diff($broken_url, $unique_element);
            self::savePageNotFound($all, $redirection_url, $redirect_type);
        } else {
            self::savePageNotFound($broken_url, $redirection_url, $redirect_type);
        }
    }

    public function getAllPnfRedirects()
    {
        $id_shop = Context::getContext()->shop->id;
        $id_shop_group = Context::getContext()->shop->id_shop_group;
        $sql = new DbQuery();
        $sql->select('*');
        $sql->from('addify_404_pages');
        $sql->where('id_shop=' . (int) $id_shop);
        $sql->where('id_shop_group=' . (int) $id_shop_group);
        $sql->orderBy('id_404_pages');

        return Db::getInstance()->executeS($sql);
    }

    public static function updateAllPnfUrls($type, $new_url)
    {
        if (!empty($type) && !empty($new_url)) {
            $id_shop = Context::getContext()->shop->id;
            $id_shop_group = Context::getContext()->shop->id_shop_group;

            return Db::getInstance()->update(
                'addify_404_pages',
                [
                    'redirect_type' => pSQL($type),
                    'redirection_url' => pSQL($new_url),
                    'id_shop' => (int) $id_shop,
                    'id_shop_group' => (int) $id_shop_group,
                ]
            );
        }
    }

    public function dropAll($rows)
    {
        foreach ($rows as $row) {
            Db::getInstance()->execute('
			DELETE FROM ' . _DB_PREFIX_ . 'addify_404_pages
			WHERE id_404_pages = ' . (int) $row);
        }
    }

    public function deleteSelectedRecord($id)
    {
        return Db::getInstance()->delete(
            'addify_404_pages',
            'id_404_pages=' . (int) $id
        );
    }

    public static function checkUriExistance($request_uri)
    {
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow('
		SELECT `redirection_url`
		FROM `' . _DB_PREFIX_ . 'addify_404_pages`
		WHERE `broken_url` = "' . pSQL($request_uri) . '"');
        if (is_array($result) && !empty($result)) {
			return $result['redirection_url'];
		}
		else {
			return false;
		}
    }

    public static function clearPnfHtaccessFile()
    {
        $path = _PS_ROOT_DIR_ . '/.htaccess';
        $content = Tools::file_get_contents($path);
        if (preg_match('#^(.*)\#begin_addify404url.*\#end_addify404url[^\n]*(.*)$#s', $content, $m)) {
            if ($write_fd = @fopen($path, 'w')) {
                fwrite($write_fd, $m[1]);
                fclose($write_fd);
            }
        }

        return true;
    }
}
