<?php
/**
 * 2019 Addify
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    Addify
 *  @copyright 2019 Addify
 *  @license   http://opensource.org/licenses/afl-3.0.php
 */
if (!defined('_PS_VERSION_')) {
    exit;
}
class AddifyRedirectUrl extends ObjectModel
{
    public $id_addifyredirecturl;
    public $redirect_type;
    public $old_uri;
    public $new_url;
    public $_join;
    public $_where;

    public static $definition = [
        'table' => 'addifyredirecturl',
        'primary' => 'id_addifyredirecturl',
        'fields' => [
            'old_uri' => ['type' => self::TYPE_STRING, 'required' => true],
            'new_url' => ['type' => self::TYPE_STRING, 'required' => true],
            'redirect_type' => ['type' => self::TYPE_STRING, 'required' => true],
        ],
    ];

    public function add($autodate = true, $null_values = false)
    {
        Shop::addTableAssociation('addifyredirecturl', ['type' => 'shop']);
        if (Shop::getContext() == Shop::CONTEXT_SHOP) {
            $this->_join .= ' LEFT JOIN `' . _DB_PREFIX_ .
            'addifyredirecturl_shop` sa ON (a.`id_addifyredirecturl` = sa.`id_addifyredirecturl` AND sa.id_shop = ' .
            (int) Context::getContext()->shop->id . ') ';
        }

        if (Shop::getContext() == Shop::CONTEXT_SHOP && Shop::isFeatureActive()) {
            $this->_where = ' AND sa.`id_shop` = ' . (int) Context::getContext()->shop->id;
        }
        if (!parent::add($autodate, $null_values)) {
            return false;
        }

        return true;
    }

    public function update($null_values = false)
    {
        if (parent::update($null_values)) {
            return true;
        }

        return false;
    }

    public static function getAllRedirects()
    {
        $id_shop = (int) Context::getContext()->shop->id;

        return Db::getInstance()->executeS('
		SELECT * FROM `' . _DB_PREFIX_ . 'addifyredirecturl` r
		LEFT JOIN `' . _DB_PREFIX_ . 'addifyredirecturl_shop` s ON (r.`id_addifyredirecturl` = s.`id_addifyredirecturl`)
		WHERE s.`id_shop` = ' . (int) $id_shop . '
		ORDER BY r.`id_addifyredirecturl`');
    }

    public static function dropAll($rows)
    {
        foreach ($rows as $row) {
            Db::getInstance()->execute('
			DELETE FROM ' . _DB_PREFIX_ . 'addifyredirecturl
			WHERE id_addifyredirecturl = ' . (int) $row);
        }
    }

    public static function checkUriExistance($request_uri)
    {
        $result = Db::getInstance(_PS_USE_SQL_SLAVE_)->getRow('
        SELECT `new_url`
        FROM `' . _DB_PREFIX_ . 'addifyredirecturl`
        WHERE `old_uri` = "/' . pSQL($request_uri) . '"');
        if (is_array($result) && !empty($result)) {
            return $result['new_url'];
        }
        else {
            return false;
        }
    }

    public static function getNewRedirectUrl($request_uri)
    {
        $id_shop = (int) Context::getContext()->shop->id;

        $results = Db::getInstance()->executeS('
        SELECT * FROM `' . _DB_PREFIX_ . 'addifyredirecturl` r
        LEFT JOIN `' . _DB_PREFIX_ . 'addifyredirecturl_shop` s ON (r.`id_addifyredirecturl` = s.`id_addifyredirecturl`)
        WHERE s.`id_shop` = ' . (int) $id_shop . '
        ORDER BY r.`id_addifyredirecturl`');

        foreach ($results as $result) {

            $old_uri = $result['old_uri'];

            $new_url = $result['new_url'];

            $current_uri = '/'.$request_uri;

            if($old_uri == $current_uri){
                return $new_url;
            }
        }

        return 0;
    }


    public static function isExists($old_uri, $id_addifyredirecturl = null)
    {
        $sql1 = new DbQuery();
        $sql1->select('id_addifyredirecturl');
        $sql1->from('addifyredirecturl');
        $result = Db::getInstance()->executeS($sql1);
        if (!empty($result)) {
            foreach ($result as $res) {
                if ($id_addifyredirecturl != $res['id_addifyredirecturl']) {
                    $id_shop = (int) Context::getContext()->shop->id;
                    $sql = new DbQuery();
                    $sql->select('*');
                    $sql->from('addifyredirecturl', 'r');
                    $sql->leftJoin('addifyredirecturl_shop', 's', 'r.`id_addifyredirecturl` = s.`id_addifyredirecturl`');
                    $sql->where('r.old_uri =   "' . pSQL($old_uri) . '" ');
                    $sql->where('s.id_shop =' . (int) $id_shop);
                    $sql->where('r.id_addifyredirecturl !=' . (int) $id_addifyredirecturl);

                    return (bool) Db::getInstance()->executeS($sql);
                }
            }
        } else {
            $id_shop = (int) Context::getContext()->shop->id;
            $sql = new DbQuery();
            $sql->select('*');
            $sql->from('addifyredirecturl', 'r');
            $sql->leftJoin('addifyredirecturl_shop', 's', 'r.`id_addifyredirecturl` = s.`id_addifyredirecturl`');
            $sql->where('r.old_uri =   "' . pSQL($old_uri) . '" ');
            $sql->where('s.id_shop =' . (int) $id_shop);

            return (bool) Db::getInstance()->executeS($sql);
        }
    }
}
