<?php
/**
 * 2019 Addify
 *
 * NOTICE OF LICENSE
 *
 * This source file is subject to the Academic Free License (AFL 3.0)
 * that is bundled with this package in the file LICENSE.txt.
 * It is also available through the world-wide-web at this URL:
 * http://opensource.org/licenses/afl-3.0.php
 * If you did not receive a copy of the license and are unable to
 * obtain it through the world-wide-web, please send an email
 * to license@prestashop.com so we can send you a copy immediately.
 *
 * DISCLAIMER
 *
 * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
 * versions in the future. If you wish to customize PrestaShop for your
 * needs please refer to http://www.prestashop.com for more information.
 *
 *  @author    Addify
 *  @copyright 2019 Addify
 *  @license   http://opensource.org/licenses/afl-3.0.php
 */
if (!defined('_PS_VERSION_')) {
    exit;
}
class AdminAddifyManageRedirectController extends ModuleAdminController
{
    public function __construct()
    {
        $this->className = 'AddifyRedirectUrl';
        $this->table = 'addifyredirecturl';
        $this->identifier = 'id_addifyredirecturl';
        $this->lang = false;
        $this->bootstrap = true;
        $this->deleted = false;
        $this->explicitSelect = true;
        $this->allow_export = true;
        $this->context = Context::getContext();
        parent::__construct();
        $this->bulk_actions = [
            'delete' => [
                'text' => $this->l('Delete selected'),
                'confirm' => $this->l('Delete selected items?'),
                'icon' => 'icon-trash',
            ],
        ];

        $this->fields_list = [
            'id_addifyredirecturl' => ['title' => $this->l('ID'), 'align' => 'center', 'width' => 30],
            'old_uri' => ['title' => $this->l('Old URI')],
            'new_url' => ['title' => $this->l('New URL')],
            'redirect_type' => ['title' => $this->l('Redirect Type'), 'align' => 'center'],
        ];
    }

    public function init()
    {
        parent::init();
        Shop::addTableAssociation($this->table, ['type' => 'shop']);
        if (Shop::getContext() == Shop::CONTEXT_SHOP) {
            $this->_join .= ' LEFT JOIN `' . _DB_PREFIX_ .
            'addifyredirecturl_shop` sa ON (a.`id_addifyredirecturl` = sa.`id_addifyredirecturl` AND sa.id_shop = ' .
            (int) $this->context->shop->id . ') ';
        }

        if (Shop::getContext() == Shop::CONTEXT_SHOP && Shop::isFeatureActive()) {
            $this->_where = ' AND sa.`id_shop` = ' . (int) Context::getContext()->shop->id;
        }
    }

    public function renderList()
    {
        $this->addRowAction('edit');
        $this->addRowAction('delete');

        $this->context->smarty->assign([
            'href' => $this->context->link->getAdminLink('AdminModules', true) . '&configure=addifyadvanceurls&tab_module=redirects&module_name=addifyadvanceurls',
        ]);
        $backButton = $this->context->smarty->fetch(dirname(__FILE__) . '/../../views/templates/admin/back_button.tpl');
        
        return parent::renderList().$backButton;
    }

    public function initToolbar()
    {
        $module_index = $this->context->link->getAdminLink('AdminModules', false);
        $token = Tools::getAdminTokenLite('AdminModules');
        $url = $module_index . '&configure=addifyadvanceurls&token=' . $token . '&tab_module=seo&module_name=addifyadvanceurls';

        parent::initToolbar();
    }

    public function renderForm()
    {
        $redirect_types = [
            ['value' => 301, 'name' => $this->l('301 Moved Permanently')],
            ['value' => 302, 'name' => $this->l('302 Moved Temporarily')],
            ['value' => 303, 'name' => $this->l('303 Other Source')],
        ];

        $this->fields_form = [
            'tinymce' => true,
            'legend' => [
                'title' => $this->l('Manage Redirects'),
                'icon' => 'icon-cog',
            ],
            'input' => [
                [
                    'type' => 'select',
                    'label' => $this->l('Redirect Type'),
                    'name' => 'redirect_type',
                    'options' => [
                        'query' => $redirect_types,
                        'id' => 'value',
                        'name' => 'name',
                    ],
                    'required' => true,
                ],
                [
                    'type' => 'text',
                    'label' => $this->l('Old URI'),
                    'name' => 'old_uri',
                    'required' => true,
                    'desc' => $this->l(
                        'Do not use http, www and domain for old URI.
                        Your old URI must start with "/" (i.e /old-uri.html)'
                    ),
                    'hint' => $this->l('Invalid characters:') . ' <>;=#{}',
                ],
                [
                    'type' => 'text',
                    'label' => $this->l('New URL'),
                    'name' => 'new_url',
                    'desc' => $this->l(
                        'Use full URL with http/https, www and domain
                        (i.e http://www.example.com/en/new-uri.html)'
                    ),
                    'required' => true,
                    'hint' => $this->l('Invalid characters:') . ' <>;=#{}',
                ],
            ],
            'submit' => [
                'title' => $this->l('Save'),
                'name' => 'submitAddifyRedirects',
            ],
        ];
        if (Shop::isFeatureActive()) {
            $this->fields_form['input'][] = [
                'type' => 'shop',
                'label' => $this->l('Shop association'),
                'name' => 'checkBoxShopAsso',
            ];
        }

        return parent::renderForm();
    }

    public function postProcess()
    {
        $obj = new AddifyRedirectUrl();
        $path = _PS_ROOT_DIR_ . '/.htaccess';
        if (Tools::isSubmit('submitAddifyRedirects')
            || Tools::isSubmit('deleteaddifyredirecturl')
            || Tools::isSubmit('submitOptionsredirects')) {
            $old_uri = pSQL(Tools::getValue('old_uri'));
            $id_addifyredirecturl = (int) Tools::getValue('id_addifyredirecturl');
            $new_url = pSQL(Tools::getValue('new_url'));

            $this->clearHtaccessFile();

            if (!empty($old_uri) && $obj->isExists($old_uri, $id_addifyredirecturl)) {
                return $this->context->controller->warnings[] = $this->l(
                    'You can not add duplicate old urls'
                );
            } elseif (!empty($new_url) && !Validate::isAbsoluteUrl($new_url)) {
                $this->context->controller->errors[] = $this->l(
                    'Please enter valid new url.'
                );
            } else {
                parent::postProcess();

                $redirects_collection = $obj->getAllRedirects();
                $old_htaccess_content = Tools::file_get_contents($path);
                if (file_exists($path)) {
                    $content = Tools::file_get_contents($path);
                    if (preg_match('#^(.*)\#begin_addifyredirecturl.*\#end_addifyredirecturl[^\n]*(.*)$#s', $content, $m)) {
                        if (!$write_fd = @fopen($path, 'w')) {
                            $this->errors[] = Tools::displayError('.htaccess not found OR not editable.');

                            return false;
                        } elseif ($write_fd = @fopen($path, 'w')) {
                            fwrite($write_fd, $m[1]);
                            fwrite($write_fd, "\n#begin_addifyredirecturl\n");

                            // RedirectMatch or Redirect
                            foreach ($redirects_collection as $redirect) {
                                if (!empty($redirect['new_url'])) {
                                    fwrite(
                                        $write_fd,
                                        'RedirectMatch ' . $redirect['redirect_type'] . ' ' .
                                        $redirect['old_uri'] . ' ' . $redirect['new_url'] . "\n"
                                    );
                                }
                            }

                            fwrite($write_fd, "#end_addifyredirecturl\n");
                            fclose($write_fd);
                        }
                    } else {
                        if (!$write_fd = @fopen($path, 'w')) {
                            $this->errors[] = Tools::displayError('.htaccess not found OR not editable.');

                            return false;
                        } elseif ($write_fd = @fopen($path, 'w')) {
                            fwrite($write_fd, $old_htaccess_content);
                            fwrite($write_fd, "\n#begin_addifyredirecturl\n");
                            foreach ($redirects_collection as $redirect) {
                                if (!empty($redirect['new_url'])) {
                                    fwrite(
                                        $write_fd,
                                        'RedirectMatch ' . $redirect['redirect_type'] . ' ' .
                                        $redirect['old_uri'] . ' ' . $redirect['new_url'] . "\n"
                                    );
                                }
                            }

                            fwrite($write_fd, "#end_addifyredirecturl\n");
                            fclose($write_fd);
                        }
                    }
                } else {
                    $this->errors[] = Tools::displayError('.htaccess not found OR not editable.');

                    return false;
                }

            }
        } elseif (Tools::isSubmit('submitBulkdeleteaddifyredirecturl')) {
            $redirects_box = Tools::getValue('addifyredirecturlBox');
            if (isset($redirects_box)) {
                $obj->dropAll($redirects_box);
                $this->clearHtaccessFile();
                $this->addUrlsToHtaccessFile();
                
            }
        } elseif (Tools::isSubmit('exportaddifyredirecturl')) {
            $this->processExport();
        } elseif (Tools::isSubmit('info')) {
            $inserted = (int) Tools::getValue('inserted');
            $invalid = (int) Tools::getValue('invalid');
            $duplicate = (int) Tools::getValue('duplicate');

            if ($inserted > 0) {
                $this->confirmations[] = $this->l('CSV data imported.');
            }

            $this->informations[] = $this->l('Duplicate and invalid records will be skipped.');
            $this->informations[] = $this->l('Inserted records : ') . $inserted;
            $this->informations[] = $this->l('Invalid records : ') . $invalid;
            $this->informations[] = $this->l('Duplicate records : ') . $duplicate;
        } elseif (Tools::isSubmit('required_fields_error')) {
            $this->errors[] = Tools::displayError('Import unsuccessfull...missing required fields.');
        }
    }

    public function clearHtaccessFile()
    {
        $path = _PS_ROOT_DIR_ . '/.htaccess';
        $content = Tools::file_get_contents($path);
        if (preg_match('#^(.*)\#begin_addifyredirecturl.*\#end_addifyredirecturl[^\n]*(.*)$#s', $content, $m)) {
            if ($write_fd = @fopen($path, 'w')) {
                fwrite($write_fd, $m[1]);
                fclose($write_fd);
            }
        }

        return true;
    }

    public function addUrlsToHtaccessFile()
    {
        $obj = new AddifyRedirectUrl();
        $path = _PS_ROOT_DIR_ . '/.htaccess';
        $redirects_collection = $obj->getAllRedirects();
        $old_htaccess_content = Tools::file_get_contents($path);
        if (file_exists($path)) {
            $content = Tools::file_get_contents($path);
            if (preg_match('#^(.*)\#begin_addifyredirecturl.*\#end_addifyredirecturl[^\n]*(.*)$#s', $content, $m)) {
                if (!$write_fd = @fopen($path, 'w')) {
                    $this->errors[] = Tools::displayError('.htaccess not found OR not editable.');

                    return false;
                } elseif ($write_fd = @fopen($path, 'w')) {
                    fwrite($write_fd, $m[1]);
                    fwrite($write_fd, "\n#begin_addifyredirecturl\n");

                    // RedirectMatch or Redirect
                    foreach ($redirects_collection as $redirect) {
                        if (!empty($redirect['new_url'])) {
                            fwrite(
                                $write_fd,
                                'RedirectMatch ' . $redirect['redirect_type'] . ' ' .
                                $redirect['old_uri'] . ' ' . $redirect['new_url'] . "\n"
                            );
                        }
                    }

                    fwrite($write_fd, "#end_addifyredirecturl\n");
                    fclose($write_fd);
                }
            } else {
                if (!$write_fd = @fopen($path, 'w')) {
                    $this->errors[] = Tools::displayError('.htaccess not found OR not editable.');

                    return false;
                } elseif ($write_fd = @fopen($path, 'w')) {
                    fwrite($write_fd, $old_htaccess_content);
                    fwrite($write_fd, "\n#begin_addifyredirecturl\n");
                    foreach ($redirects_collection as $redirect) {
                        if (!empty($redirect['new_url'])) {
                            fwrite(
                                $write_fd,
                                'RedirectMatch ' . $redirect['redirect_type'] . ' ' .
                                $redirect['old_uri'] . ' ' . $redirect['new_url'] . "\n"
                            );
                        }
                    }

                    fwrite($write_fd, "#end_addifyredirecturl\n");
                    fclose($write_fd);
                }
            }
        } 
    }
}
